/*
 * The contents of this file are copyright 2012 Clinton Bale
 * <clinton.bale@gmail.com>
 *
 * This work is licensed under the Creative Commons "Attribution-Share Alike 3.0 Unported" License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/3.0/legalcode
 * or, send a letter to Creative Commons, 171 2nd Street, Suite 300, San Francisco, California, 94105, USA.
*/

#include "mapfile.h"
#include <cassert>

MapFile::MapFile(const char* fileName)
{	
	fp = fopen(fileName, "w+");
}

MapFile::~MapFile()
{
	if(fp) {
		fclose(fp);
	}
}

bool MapFile::create() {
	if(fp) {
		fprintf(fp, "{\n\"classname\" \"worldspawn\"\n\"mapversion\" \"%d\"\n\"skyname\" \"2desert\"\n\"MaxRange\" \"10240\"\n", MAP_VERSION);
		return true;
	}
	return false;
}

bool MapFile::close()
{
	if(fp) {
		fprintf(fp, "}\n");
		return !fclose(fp);
	}
	return false;
}

void MapFile::write_face(face_t side, int x, int y, int z, int size, int height, const char* texture)
{
	this->write_face(side, x, y, z, size, size, height, texture);
}

void MapFile::write_face(face_t side, int x, int y, int z, int length, int width, int height, const char* texture)
{
	switch(side) {
	case TOP:
		fprintf(fp, "(%d %d %d) (%d %d %d) (%d %d %d) %s [ 1 0 0 0 ] [ 0 -1 0 0 ] 0 1 1\n",
			x, y, z+height,
			x, y+length, z+height,
			x+width, y+length, z+height,
			texture);
		break;
	case BOTTOM:
		fprintf(fp, "(%d %d %d) (%d %d %d) (%d %d %d) %s [ 1 0 0 0 ] [ 0 -1 0 0 ] 0 1 1\n",
			x, y+length, z,
			x, y, z,
			x+width, y, z,
			texture);
		break;
	case LEFT:
		fprintf(fp, "(%d %d %d) (%d %d %d) (%d %d %d) %s [ 0 1 0 0 ] [ 0 0 -1 0 ] 0 1 1\n",
			x, y, z,
			x, y+length, z,
			x, y+length, z+height,
			texture);
		break;
	case RIGHT:
		fprintf(fp, "(%d %d %d) (%d %d %d) (%d %d %d) %s [ 0 1 0 0 ] [ 0 0 -1 0 ] 0 1 1\n",
			x+width, y+length, z,
			x+width, y, z,
			x+width, y, z+height,
			texture);
		break;
	case FRONT:
		fprintf(fp, "(%d %d %d) (%d %d %d) (%d %d %d) %s [ 1 0 0 0 ] [ 0 0 -1 0 ] 0 1 1\n",
			x, y+length, z,
			x+width, y+length, z,
			x+width, y+length, z+height,
			texture);
		break;
	case BACK:
		fprintf(fp, "(%d %d %d) (%d %d %d) (%d %d %d) %s [ 1 0 0 0 ] [ 0 0 -1 0 ] 0 1 1\n",
			x+width, y, z,
			x, y, z,
			x, y, z+height,
			texture);
		break;
	}
}

void MapFile::create_brush(int x, int y, int z, int length, int width, int height, const char* texture, int nullFaces)
{
	assert(fp != NULL);

	fprintf(fp, "{\n");
	for(int i = 0; i < 6; i++) {		
		this->write_face((face_t)(1<<i), x, y, z, length, width, height, ((nullFaces) & (1<<(i))) ? "NULL" : texture);
	}
	fprintf(fp, "}\n");
}