/*
 * The contents of this file are copyright 2012 Clinton Bale
 * <clinton.bale@gmail.com>
 *
 * This work is licensed under the Creative Commons "Attribution-Share Alike 3.0 Unported" License.
 * To view a copy of this license, visit http://creativecommons.org/licenses/by-sa/3.0/legalcode
 * or, send a letter to Creative Commons, 171 2nd Street, Suite 300, San Francisco, California, 94105, USA.
*/

#pragma once
#ifndef MAZE_GEN_H
#define MAZE_GEN_H

#include <stack>
#include <vector>
#include <cassert>

enum direction_t {
	NONE = 0,
	NORTH = 1,
	SOUTH = 2,
	WEST = 4,
	EAST = 8,
	ALL = 0xF
};

struct cell_t
{
	size_t walls : 4;
	int x;
	int y;

	direction_t get_wall_bit(const cell_t* cell)
    {
		if(cell->x == x)		
			return cell->y < y ? NORTH : SOUTH;
		return cell->x < x ? WEST : EAST;
    }

	void remove_wall(cell_t* cell)
	{
		direction_t wallBit = get_wall_bit(cell);
		direction_t opposite = NONE;			
		
		//tkg!
		opposite = (direction_t)( ((wallBit & 10) >> 1) | ((wallBit & 5) << 1) );

		walls &= ~(wallBit);		
		cell->walls &= ~(opposite);		
	}
};

class MazeGenerator
{
private:	
	cell_t** maze;
	std::stack<void*> garbage;

	size_t width;
	size_t height;

	bool get_neighbors(int xpos, int ypos, std::vector<cell_t*>& out);
	void initialize(void);
public:
	MazeGenerator(unsigned int w, unsigned int h);
	~MazeGenerator(void);

	size_t get_width() const {
		return width;
	}
	size_t get_height() const {
		return height;
	}
	size_t total_cells() const {
		return width * height;
	}

	void generate(int, int);
	void generate(void);	
	void print();
	char** print(bool, int*, int*);
};

#endif